

export interface returnCommonPropsPromise {
  status: boolean;
  msg: string;
}

// export interface returnGetDataSqliteT extends Partial<Pick<returnCommonPropsPromise, 'msg'>>, Omit<returnCommonPropsPromise, 'msg'>{
export interface returnGetDataSqliteT extends returnCommonPropsPromise{
  values:[] | {}[];
}
export interface returnGetDataSLStoreT extends returnCommonPropsPromise{
  data: {}
}

export interface SelectWhere{
  sql?: string;
}



export interface Params {
  where?:object;
  whereKey?: {[key:string]: string[] | undefined};
  ignoreWhere?: {[key:string]: string[] | undefined};
  condition?: 'AND' | 'OR';
  stringWhere?:string | undefined;
}

export interface UpdateWhere extends Omit<Params, 'whereKey' | 'ignoreWhere'>{ }

export interface setDataSqliteOptions {
  isCreateDate?: boolean | undefined;
  rewriteTable?: boolean | undefined;
}


export type commonFunctionSqliteT = (connect:SQLitePlugin, nameTable?:string) => Promise<returnCommonPropsPromise> 
export type getDataSqliteT = (connect:SQLitePlugin, nameTable:string, params:Params) => Promise<returnGetDataSqliteT> 
export type setDataSqliteT = (connect:SQLitePlugin, nameTable:string, payload: object, options?:setDataSqliteOptions) => Promise<returnCommonPropsPromise> 
export type updateDataSqliteT = (connect:SQLitePlugin, nameTable:string, payload: object, {where, condition, stringWhere}: UpdateWhere) => Promise<returnCommonPropsPromise> 
export type remoteDataSqliteT = (connect:SQLitePlugin, nameTable:string, params:Params) => Promise<returnCommonPropsPromise> 

/*#############----------<{ Class Sqlite }>-----------########## */
export type commonMethodSqliteT = (nameTable?:string) => Promise<returnCommonPropsPromise> 
export type getDataT = (nameTable:string, params:Params) => Promise<returnGetDataSqliteT> 
export type setDataT = (nameTable:string, payload: object, options?:setDataSqliteOptions) => Promise<returnCommonPropsPromise> 
export type updateDataT = (nameTable:string, payload: object, {where, condition, stringWhere}: UpdateWhere) => Promise<returnCommonPropsPromise> 
export type remoteDataT = (nameTable:string, params:Params) => Promise<returnCommonPropsPromise> 


/*#############----------<{ Class SLStore }>-----------########## */
export interface getItemsI {
  ignoreKeys: string[]
}
// export type commonMethodSqliteT = (nameTable:string) => Promise<returnCommonPropsPromise> 
export type getItemT = (key:string) => Promise<returnGetDataSLStoreT> 
export type getItemsT = (options?:getItemsI) => Promise<returnGetDataSLStoreT> 

export type setItemT = (key:string, value: any, options?:setDataSqliteOptions) => Promise<returnCommonPropsPromise> 
// export type updateDataT = (nameTable:string, payload: object, {where, condition, stringWhere}: UpdateWhere) => Promise<returnCommonPropsPromise> 
export type remoteItemT = ({keys, ignoreKey}:{keys?:string[], ignoreKey?:string[]}) => Promise<returnCommonPropsPromise> 




export type temporary = any | void;


//db - SQLitePlugin (Экземпляр openDatabase)
export interface SQLitePlugin {
  readonly dbname: string
  openError: (e: any) => temporary
  openSuccess: (e: any) => temporary
  readonly openargs: {
      name?: string,
      location?: string,
      androidDatabaseProvider?: string,
      dblocation?: string,
      androidOldDatabaseImplementation?: number
  }
  
  abortAllPendingTransactions?: () => temporary
  addTransaction: (t: any) => temporary
  close: (success?: successCheckedType, error?: errorCheckedType) => temporary
  open: (success?: (db: SQLitePlugin) => void | any, error?: (err: Error | any) => void | any) => temporary
  databaseFeatures?: {isSQLitePluginDatabase: boolean} 
  executeSql: (
    statement: string,//SQL строка
    params?: any[],
    success?: (res: IResourceSuccess) => temporary,
    error?: (err: IResourceError) => temporary
  ) => temporary
  openDBs: any//{'arm.db': "OPEN" | string}
  readTransaction: (
    fn: fnTransitionType,
    error: errorCheckedType,
    success: successCheckedType
  ) => temporary
  sqlBatch: (
    sqlStatements: any[],
    /*
      [
        'CREATE TABLE MyTable IF NOT EXISTS (name STRING, balance INTEGER)',
        [ 'INSERT INTO MyTable VALUES (?1,?2)', ['Alice', 100] ],
        [ 'INSERT INTO MyTable VALUES (?1,?2)', ['Betty', 200] ],
      ]
    */
    success: successCheckedType,
    error: errorCheckedType
  ) => temporary
  startNextTransaction: () => temporary
  transaction: (
    fn: fnTransitionType,
    error?: errorCheckedType,
    success?: successCheckedType
  ) => void
}

//tx
export interface SQLitePluginTransaction {
  db: SQLitePlugin
  error: any
  executes: []
  finalized: boolean
  fn: (tx, res)=> temporary
  readOnly: boolean
  success: any
  txlock: boolean

  abort: (txFailure: temporary) => temporary
  abortFromQ: (sqlerror: temporary) => temporary
  addStatement: (sql: string, values?: any[], success?:SuccessTxAndResource, error?: ErrorTxAndErr) => temporary
  executeSql: (sql: string, values?: any[], success?:SuccessTxAndResource, error?: ErrorTxAndErr) => temporary
  finish: () => temporary
  handleStatementFailure: (handler: temporary, response: temporary) => temporary
  handleStatementSuccess: (handler: temporary, response: temporary) => temporary
  run: () => temporary
  start: () => temporary
}

export interface IResourceSuccess {
  readonly insertId: number | undefined
  rows: {item: (i: number) => any, readonly length: number }
  readonly rowsAffected: number
}

export interface IResourceError {
  readonly code: number
  readonly message: string
}

type fnTransitionType = (tx: SQLitePluginTransaction) => temporary

type successCheckedType = () => temporary
type errorCheckedType = (err: Error ) => temporary

type SuccessTxAndResource = (tx: SQLitePluginTransaction, res: IResourceSuccess) => temporary
type ErrorTxAndErr = (tx: SQLitePluginTransaction, err: IResourceError) => temporary



export interface openDatabaseOptionType{
  name: string,
  location: 'default',
  androidDatabaseProvider?: 'system'
}


export interface sqlitePluginType {
  openDatabase: (param:openDatabaseOptionType, success?: (db: SQLitePlugin) => void, error?: errorCheckedType) => SQLitePlugin; 
  deleteDatabase: (first: {name: string, location: 'default'}, success: successCheckedType, error: errorCheckedType) => void
  echoTest: (okcb, errorcb) => any | void; 
  selfTest: (successcb, errorcb) => temporary 
  sqliteFeatures?: {isSQLitePlugin: boolean}
}



 declare global {
  interface Window {
    db: any;
    sqlitePlugin: sqlitePluginType;
    cordova: any;
  }
 }


